/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.processor.processor;

import java.net.InetSocketAddress;
import java.net.ProxySelector;
import java.net.http.HttpClient;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.client.reactive.ClientHttpConnector;
import org.springframework.http.client.reactive.JdkClientHttpConnector;
import org.springframework.web.reactive.function.client.WebClient;

import de.ozgcloud.processor.ProcessorManagerConfiguration;
import lombok.RequiredArgsConstructor;

@Configuration
@RequiredArgsConstructor
class ProcessorConfiguration {

	@Qualifier(ProcessorManagerConfiguration.PROXY_PROPERTIES_NAME) // NOSONAR
	private final Optional<ProxyProperties> proxyProperties;

	@Bean
	WebClient webClient(WebClient.Builder webClientBuilder) {
		proxyProperties.map(this::createProxyConnector).ifPresent(webClientBuilder::clientConnector);
		return webClientBuilder.build();
	}

	ClientHttpConnector createProxyConnector(ProxyProperties httpProxy) {
		var addr = new InetSocketAddress(httpProxy.getHost(), httpProxy.getPort());
		var httpClient = HttpClient.newBuilder().proxy(ProxySelector.of(addr)).build();
		return new JdkClientHttpConnector(httpClient);
	}
}
